
#include <assert.h>
#include <iostream>
#include <fstream>
#include "timer.hxx"
#include "cuda_check.cuh"
 
template <typename T>
__global__ void stride(T* a, int s)
{
  int i = (blockDim.x * blockIdx.x + threadIdx.x) * s;
  a[i] = a[i] + 1;
}
 
template <typename T>
void runTest(long n, const char *type, long maxStride)
{
  std::string s = "res_";
  s += type;
  s += ".txt";
  std::ofstream f(s.c_str());

  int blockSize = 256;
 
  T *d_a;
     
  CUDA_CHECK_OP( cudaMalloc(&d_a, n * maxStride * sizeof(T)) );
  std::cout <<  "Vector of " << n * maxStride << " " <<type << std::endl;

  cudaMemset(d_a, 1, n * sizeof(T));

  Timer Tm;

//  Tm.start();

  stride<<<n/blockSize, blockSize>>>(d_a, 1);
  cudaDeviceSynchronize();

//  Tm.stop();
//  f << 0 << " " << Tm.elapsed() << std::endl;

//  Tm.restart();

  for (int i = 1; i <= maxStride; i++) {

    Tm.restart();

    stride<<<n/blockSize, blockSize>>>(d_a, i);
    cudaDeviceSynchronize();

    Tm.stop();
    f << i << " " << Tm.elapsed() << std::endl;
  }
  f << std::endl;
 
  cudaFree(d_a);
 }
 
 int main(int argc, char **argv)
 {   
  int nMB = argc > 1 ? atol(argv[1]) : 4;
  int maxStride = argc > 1 ? atol(argv[2]) : 10;
  int deviceId = 0;
  cudaDeviceProp prop;
   
  CUDA_CHECK_OP( cudaSetDevice(deviceId) )   ;
  CUDA_CHECK_OP( cudaGetDeviceProperties(&prop, deviceId) );
  std::cout << "Device:             " << prop.name << std::endl;

  long n = nMB*1024L*1024L;
  std::cout << "Number of threads : " << n << " threads" << std::endl;

  runTest<double>(n, "doubles", maxStride);
  runTest<float> (n, "floats", maxStride);
  runTest<int>   (n, "ints", maxStride);

  return 0;
}
