#! /usr/bin/env python

import os, sys, subprocess, argparse, re

parser = argparse.ArgumentParser()
parser.add_argument('--level')
parser.add_argument('--dryrun', action='store_true')
args = parser.parse_args()

if not os.path.exists('kokkos'):
    subprocess.call(['git', 'clone', 'https://github.com/kokkos/kokkos'])

with open("kokkos/bin/nvcc_wrapper") as fIn:
    s = fIn.read()

pattern = 'default_arch="sm_.."'
repl = 'default_arch="sm_' + args.level+ '"'

t = re.sub(pattern, repl, s)
if not t == s:
    with open("kokkos/bin/nvcc_wrapper", "w") as fOut:
        fOut.write(t)

baseDir = os.getcwd()
srcDir = os.path.join(baseDir, 'kokkos')

for v in ['Serial', 'OpenMP', 'Cuda']:
    print('\nbuild kokkos version ' + v)
    e = os.environ.copy()
    buildDir = os.path.join(baseDir, 'kokkos', 'build', v)
    installDir = os.path.join(baseDir, 'kokkos', 'install', v)
    configureCmd = ['cmake']
    configureCmd.append('-DKokkos_ENABLE_' + v.upper() + '=ON')

    if not v == 'Serial':
        configureCmd.append('-DKokkos_ENABLE_SERIAL=ON')
    if v == 'Cuda':
        configureCmd.append('-DKokkos_ENABLE_CUDA_LAMBDA=ON')

    configureCmd.append('-DCMAKE_INSTALL_PREFIX=' + installDir)
    configureCmd.append('-DCMAKE_INSTALL_MESSAGE=LAZY')
    configureCmd.append(srcDir)
    compileCmd = ['make', '-j', '4', 'install']

    print('\n\t'.join(configureCmd))
    print('\n' + ' '.join(compileCmd))

    if not args.dryrun:
        if not os.path.exists(buildDir):
            os.makedirs(buildDir)

        subprocess.call(configureCmd, env=e, cwd=buildDir)
        subprocess.call(compileCmd, cwd=buildDir, env=e)
