#!  /usr/bin/env python3

import os, sys, subprocess, argparse, platform, shutil

def manage():
  
  p = platform.system()
  if p == 'Windows':
    gen = '-GNinja'
    compileCmd = ['ninja', 'install']
  else:
    gen = '-GUnix Makefiles'
    compileCmd = ['make', '--no-print-directory', 'install']

  baseDir = os.getcwd()
  srcDir = os.path.join(baseDir, 'src')

  for v in ['Serial', 'OpenMP', 'Threads', 'Cuda']:

    err = True
    kokkosDir = os.path.realpath(
        os.path.join(baseDir, '..', 'kokkos', 'install', v))

    for l in ['lib', 'lib64']:
      kokkosCmakeConfig = os.path.join(kokkosDir, l, 'cmake', 'Kokkos')
      if os.path.exists(kokkosCmakeConfig):
        err = False
        break

    if err:
      sys.exit(-1)

    e = os.environ.copy()
    buildDir = os.path.join(baseDir, 'build', v)
    installDir = os.path.join(baseDir, 'install', v)
 
    cmake_params = []
    cmake_params.append('-DVERSION=' + v)
    cmake_params.append('-DCMAKE_CXX_EXTENSIONS=Off')
    cmake_params.append('-DCMAKE_INSTALL_PREFIX=' + installDir)
    cmake_params.append('-DCMAKE_PREFIX_PATH=' + kokkosCmakeConfig) 
    cmake_params.append(gen)

    if not os.path.exists(buildDir):
      os.makedirs(buildDir)
  
    configureCmd = ['cmake'] + cmake_params + [srcDir]

    err = subprocess.call(configureCmd, cwd=buildDir, env=e)

    if err == 0:
      err = subprocess.call(compileCmd, cwd=buildDir, env=e)

if __name__ == '__main__':
    manage()

