#include <iostream>
#include "calcul_Cuda.hxx"
#include "timer.hxx"
#include "cuda_check.cuh"
#include <cstdlib>
#include <cuda.h>
#include <cuda_runtime.h>
#include <stdio.h>

const size_t Mb = 1048576;

void ShowMemory()
{
  size_t mfree, mtotal;
  cudaMemGetInfo (&mfree, &mtotal);
  std::cout << "\navailable memory " << mfree/Mb << " megabytes";
}

void GetProperties(Properties &p)
{
  double f;
  const char * factor = getenv("FACTOR");
  if (factor)
     f = atof(factor);
  else
     f = 0.5;

  size_t mfree, mtotal;
  cudaMemGetInfo (&mfree, &mtotal);
  std::cout << "\navailable memory " << mfree/Mb << " megabytes (total " << mtotal/Mb << " megabytes)\n" << std::endl;
  p.Nmax = mfree/(3*sizeof(double)) * f;
 
  int value;
  cudaDeviceGetAttribute ( &value, cudaDevAttrMaxBlockDimX, 0);
  p.BlockMax = value;
   std::cout << "max. block size = " << p.BlockMax << std::endl;
}

Calcul_Cuda::Calcul_Cuda(Properties & p) : _p(p)
{
  n = _p.N; 
  std::cout << "vector size : " << _p.N << " (max: " << _p.Nmax/1000000 << " millions(s))\n" << std::endl;

  size_t bytes = n*sizeof(double);
  ShowMemory();

  CUDA_CHECK_OP(cudaMalloc(&d_u, bytes));
  ShowMemory();

  CUDA_CHECK_OP(cudaMalloc(&d_w, bytes));
  ShowMemory();

  CUDA_CHECK_OP(cudaMalloc(&d_v, bytes));
  ShowMemory();

  std::cout << std::endl;
}

Calcul_Cuda::~Calcul_Cuda()
{
  ShowMemory();
  cudaFree(d_u);

  ShowMemory();
  cudaFree(d_v);

  ShowMemory();
  cudaFree(d_w);

  ShowMemory();
  cudaDeviceSynchronize();

}

__global__ void vecInit(double *a, double *b, size_t n)
{
  size_t id = size_t(blockIdx.x)*blockDim.x+threadIdx.x;
  double x;
  
  if (id < n)
  {
    x = 1.0 * id;
    a[id] = sin(x)*sin(x);
    b[id] = cos(x)*cos(x);
  }
}

__global__ void printTest()
{
   printf("ok\n");
}

void test()
{
  printTest<<<1, 1>>>();
  cudaDeviceSynchronize();
  CUDA_CHECK_KERNEL();
}

void Calcul_Cuda::init()
{
  cudaDeviceSynchronize();
  CUDA_CHECK_KERNEL();
  blockSize = _p.BlockDim;
  gridSize = (unsigned int) ceil((double)n/blockSize);
  _p.nBlocks = gridSize; 

  vecInit<<<gridSize, blockSize>>>(d_u, d_v, n);
  cudaDeviceSynchronize();
  CUDA_CHECK_KERNEL();
}

__global__ void vecAdd(double *c, double *a, double *b, size_t n)
{
  size_t id = blockIdx.x*blockDim.x+threadIdx.x;
  
  if (id < n)
    c[id] = a[id] + b[id];
}

void Calcul_Cuda::addition()
{  
  vecAdd<<<gridSize, blockSize>>>(d_w, d_u, d_v, n);
  cudaDeviceSynchronize();
  CUDA_CHECK_KERNEL();
}

#include <vector>
double Calcul_Cuda::verification()
{  
  size_t bytes = n*sizeof(double);
  std::vector<double> w(n);
  cudaMemcpy(w.data(), d_w, bytes, cudaMemcpyDeviceToHost);

  double s = 0;
  for (size_t i=0; i<n; i++)
    s = s + w[i];
  
  s = s/n - 1.0;
  
  return s;
}

