#include <iostream>


#include <thrust/device_vector.h>
#include <thrust/sort.h>
#include <thrust/copy.h>

#include <vector>
#include <algorithm>

#include <cstdlib>
#include <iomanip>

#include "timer.hxx"


void print_first(const char *message, const std::vector<int> & v, size_t m = 10)
{
  auto print = [](const int& k) { std::cout << " " << k; };
  if (v.size() <= m) { 
    std::cout << std::setw(20) << message << ": ";
    std::for_each(v.begin(), v.end(), print);
    std::cout << std::endl;
  }
}

void calcul_cpu   (const std::vector<int> & h_v, 
                         std::vector<int> & h_w)
{
  Timer T_cpu; T_cpu.start();
  Timer T_calcul;
  {
    h_w = h_v;

    T_calcul.start();
    std::sort(h_w.begin(), h_w.end());
    T_calcul.stop();
  }
  T_cpu.stop();
  std::cout << "N: " << std::setw(15) << h_v.size() << " Temps CPU:          " << std::setw(15) << T_cpu.elapsed() 
            << " s, dont calcul: " << T_calcul.elapsed() << " s" << std::endl;
  print_first("tri (cpu)", h_w);
}

void calcul_thrust(const std::vector<int> & h_v, 
                         std::vector<int> & h_w)
{
  Timer T_thrust; T_thrust.start();
  Timer T_calcul;
  {
    thrust::device_vector<int> d_v = h_v;

    T_calcul.start();
    thrust::sort(d_v.begin(), d_v.end());
    T_calcul.stop();

    thrust::copy(d_v.begin(), d_v.end(), h_w.begin());
  }
  T_thrust.stop();
  std::cout << "N: " << std::setw(15) << h_v.size() << " Temps GPU(Thrust): " << std::setw(15) << T_thrust.elapsed() 
            << " s, dont calcul: " << T_calcul.elapsed() << " s" << std::endl;
  print_first("tri (thrust)", h_w);
}


bool erreur(const std::vector<int> & h_v, 
            const std::vector<int> & h_w)
{
  auto it = h_v.begin();
  auto jt = h_w.begin();
  for (; it != h_v.end(); it++, jt++)
      if (*it != *jt) return true;
  return false;
}

int main(int argc, char **argv)
{
  size_t n = argc > 1 ? atol(argv[1]) : 20;

  std::vector<int> v(n), w1(n), w2(n);
  std::generate(v.begin(), v.end(), rand);
  print_first("avant", v);

  calcul_cpu(v, w1);
  calcul_thrust(v, w2);
  
  std::cout << "erreur:" << erreur(w1, w2) << std::endl;

  return 0;
}
